/*global location */
sap.ui.define([
	"kodyaz/com/controller/BaseController",
	"sap/ui/model/json/JSONModel",
	"kodyaz/com/model/formatter",
	"sap/m/MessageToast",
	"sap/ui/unified/FileUploader"
], function (BaseController, JSONModel, formatter, MessageToast, FileUploader) {
	"use strict";
	return BaseController.extend("kodyaz.com.controller.Detail", {
		formatter: formatter,
		/* =========================================================== */
		/* lifecycle methods                                           */
		/* =========================================================== */
onInit: function () {
			// Model used to manipulate control states. The chosen values make sure,
			// detail page is busy indication immediately so there is no break in
			// between the busy indication for loading the view's meta data
var oViewModel = new JSONModel({
				busy: false,
				delay: 0,
				lineItemListTitle: this.getResourceBundle().getText("detailLineItemTableHeading")
			});
			this.getRouter().getRoute("object").attachPatternMatched(this._onObjectMatched, this);
			this.setModel(oViewModel, "detailView");
			this.getOwnerComponent().getModel().metadataLoaded().then(this._onMetadataLoaded.bind(this));
		},
		/* =========================================================== */
		/* event handlers                                              */
		/* =========================================================== */
		/**
		 * Event handler when the share by E-Mail button has been clicked
		 * @public
		 */
onShareEmailPress: function () {
			var oViewModel = this.getModel("detailView");
			sap.m.URLHelper.triggerEmail(null, oViewModel.getProperty("/shareSendEmailSubject"), oViewModel.getProperty("/shareSendEmailMessage"));
		},
		/**
		 * Updates the item count within the line item table's header
		 * @param {object} oEvent an event containing the total number of items in the list
		 * @private
		 */
onListUpdateFinished: function (oEvent) {
			var sTitle, iTotalItems = oEvent.getParameter("total"), oViewModel = this.getModel("detailView");
			if (this.byId("projectMockupsList").getBinding("items").isLengthFinal()) {
				if (iTotalItems) {
					sTitle = this.getResourceBundle().getText("detailLineItemTableTitle", [iTotalItems]);
				} else {
					sTitle = this.getResourceBundle().getText("detailLineItemTableTitle");
				}
				oViewModel.setProperty("/lineItemListTitle", sTitle);
			}
		},
		/* =========================================================== */
		/* begin: internal methods                                     */
		/* =========================================================== */
		/**
		 * Binds the view to the object path and expands the aggregated line items.
		 * @function
		 * @param {sap.ui.base.Event} oEvent pattern match event in route 'object'
		 * @private
		 */
_onObjectMatched: function (oEvent) {
			var sObjectId = oEvent.getParameter("arguments").objectId;
			this.getModel().metadataLoaded().then(function () {
				var sObjectPath = this.getModel().createKey("ProjectMainSet", { ProjectKey: sObjectId });
				this._bindView("/" + sObjectPath);
			}.bind(this));
		},
		/**
		 * Binds the view to the object path. Makes sure that detail view displays
		 * a busy indicator while data for the corresponding element binding is loaded.
		 * @function
		 * @param {string} sObjectPath path to the object to be bound to the view.
		 * @private
		 */
_bindView: function (sObjectPath) {
			// Set busy indicator during view binding
var oViewModel = this.getModel("detailView");
			// If the view was not bound yet its not busy, only if the binding requests data it is set to busy again
oViewModel.setProperty("/busy", false);
			this.getView().bindElement({
				path: sObjectPath,
				events: {
					change: this._onBindingChange.bind(this),
					dataRequested: function () {
						oViewModel.setProperty("/busy", true);
					},
					dataReceived: function () {
						oViewModel.setProperty("/busy", false);
					}
				}
			});
		},
		_onBindingChange: function () {
			var oView = this.getView(), oElementBinding = oView.getElementBinding();
			// No data for the binding
if (!oElementBinding.getBoundContext()) {
				this.getRouter().getTargets().display("detailObjectNotFound");
				// if object could not be found, the selection in the master list
				// does not make sense anymore.
this.getOwnerComponent().oListSelector.clearMasterListSelection();
				return;
			}
			var sPath = oElementBinding.getPath(), oResourceBundle = this.getResourceBundle(), oObject = oView.getModel().getObject(sPath), sObjectId = oObject.ProjectKey, sObjectName = oObject.ProjectName, oViewModel = this.getModel("detailView");
			this.getOwnerComponent().oListSelector.selectAListItem(sPath);
			oViewModel.setProperty("/shareSendEmailSubject", oResourceBundle.getText("shareSendEmailObjectSubject", [sObjectId]));
			oViewModel.setProperty("/shareSendEmailMessage", oResourceBundle.getText("shareSendEmailObjectMessage", [
				sObjectName,
				sObjectId,
				location.href
			]));
		},
		_onMetadataLoaded: function () {
			// Store original busy indicator delay for the detail view
var iOriginalViewBusyDelay = this.getView().getBusyIndicatorDelay(), oViewModel = this.getModel("detailView"), oLineItemTable = this.byId("lineItemsList"), iOriginalLineItemTableBusyDelay = oLineItemTable.getBusyIndicatorDelay();
			// Make sure busy indicator is displayed immediately when
			// detail view is displayed for the first time
oViewModel.setProperty("/delay", 0);
			oViewModel.setProperty("/lineItemTableDelay", 0);
			oLineItemTable.attachEventOnce("updateFinished", function () {
				// Restore original busy indicator delay for line item table
oViewModel.setProperty("/lineItemTableDelay", iOriginalLineItemTableBusyDelay);
			});
			// Binding the view will set it to not busy - so the view is always busy if it is not bound
oViewModel.setProperty("/busy", true);
			// Restore original busy indicator delay for the detail view
oViewModel.setProperty("/delay", iOriginalViewBusyDelay);
		},
		launchURL: function (sURL) {
			// open URL in new browser tab
var win = window.open(sURL, "_blank");
			win.focus();
		},
		launchStudyURL: function () {
			// check Study URL, if it has value open in new browser tab
var lvUrl = document.getElementById("__component0---detail--studyurl-inner").value;
			if (lvUrl !== "") {
				this.launchURL(lvUrl);
			} else {
				return false;
			}
		},
		mockupCarouselPageChanged: function () {
		},
		OnSaveProject: function () {
			var msg = this.getResourceBundle().getText("detailSaved");
			var oDialogSave = new sap.m.Dialog({
				title: "Save Changes",
				type: "Message",
				state: "Success",
				contentWidth: "400px",
				contentHeight: "150px",
				content: new sap.m.Label({
					width: "100%",
					text: msg,
					textAlign: "Center"
				}),
				beginButton: new sap.m.Button({
					text: "OK",
					type: "Accept",
					width: "100px",
					icon: "sap-icon://accept",
					press: function () {
						oDialogSave.close();
					}
				}),
				afterClose: function () {
					oDialogSave.destroy();
				}
			});
			oDialogSave.open();
		},
		OnCancelProject: function () {
			var msg = this.getResourceBundle().getText("detailCancelled");
			var oDialogCancel = new sap.m.Dialog({
				title: "Cancel Changes",
				type: "Message",
				state: "Warning",
				contentWidth: "400px",
				contentHeight: "150px",
				content: new sap.m.Label({
					width: "100%",
					text: msg,
					textAlign: "Center"
				}),
				beginButton: new sap.m.Button({
					text: "Close",
					type: "Reject",
					width: "100px",
					icon: "sap-icon://sys-cancel",
					press: function () {
						oDialogCancel.close();
					}
				}),
				afterClose: function () {
					oDialogCancel.destroy();
				}
			});
			oDialogCancel.open();
		},
		onMockupUpload: function () {
			var text1 = this.getResourceBundle().getText("detailUploadMockupText");
			this.onUpload(text1);
		},
		onJourneyUpload: function () {
			var text1 = this.getResourceBundle().getText("detailUploadJourneyText");
			this.onUpload(text1);
		},
		onUpload: function (text1) {
			var title = this.getResourceBundle().getText("detailUploadFile");
			var text2 = "Choose file:";
			var oDialogUpload = new sap.m.Dialog({
				title: title,
				type: "Message",
				contentWidth: "600px",
				contentHeight: "200px",
				content: [
					new sap.m.Label({
						width: "100%",
						text: text1,
						textAlign: "Left"
					}),
					new sap.m.Label({
						width: "100px",
						text: text2,
						textAlign: "Left"
					}),
					new sap.ui.unified.FileUploader({
						id: "mockupFileUploader",
						name: "mockupFileUpload",
						uploadUrl: "../localService/mockdata/resources/",
						uploadComplete: "handleUploadComplete",
						width: "400px",
						textAlign: "Left",
						tooltip: "Choose and upload new mockup file"
					})
				],
				buttons: [
					new sap.m.Button({
						text: "Cancel",
						type: "Reject",
						width: "100px",
						icon: "sap-icon://sys-cancel",
						press: function () {
							oDialogUpload.close();
						}
					}),
					new sap.m.Button({
						text: "Close",
						type: "Emphasized",
						width: "100px",
						press: function () {
							oDialogUpload.close();
						}
					}),
					new sap.m.Button({
						text: "Upload",
						type: "Accept",
						width: "100px",
						icon: "sap-icon://upload",
						press: function () {
							oDialogUpload.close();
							MessageToast.show("File uploaded successfully");
						}
					})
				],
				afterClose: function () {
					oDialogUpload.destroy();
				}
			});
			oDialogUpload.open();
		},
		handleUploadPress: function (oEvent) {
			MessageToast.show("File uploaded successfully");	//var oFileUploader = this.getView().byId("mockupFileUploader");
                                                    //oFileUploader.upload();
		},
		handleUploadComplete: function (oEvent) {
			var sResponse = oEvent.getParameter("response");
			if (sResponse) {
				var sMsg = "";
				var m = /^\[(\d\d\d)\]:(.*)$/.exec(sResponse);
				if (m[1] == "200") {
					sMsg = "Return Code: " + m[1] + "\n" + m[2], "SUCCESS", "Upload Success";
					oEvent.getSource().setValue("");
				} else {
					sMsg = "Return Code: " + m[1] + "\n" + m[2], "ERROR", "Upload Error";
				}
				MessageToast.show(sMsg);
			}
		},
		onImageClicked: function (oEvent) {
			var title = oEvent.oSource.mProperties.alt;
			var sId = oEvent.mParameters.id + "_orj";
			var mSettings;
			var oDialogUpload = new sap.m.Dialog({
				title: title,
				type: "Message",
				content: [new sap.m.Image({ src: oEvent.oSource.mBindingInfos.src.binding.oValue })],
				buttons: [new sap.m.Button({
						text: "Close",
						type: "Emphasized",
						icon: "sap-icon://sys-cancel",
						width: "100px",
						press: function () {
							oDialogUpload.close();
						}
					})],
				afterClose: function () {
					oDialogUpload.destroy();
				}
			});
			oDialogUpload.open();
		},
        onProjectCompleted: function (oEvent) {
            if(oEvent.oSource.mProperties.selected === true){
                this.showProjectCloseMsg(oEvent.mParameters.id);
            }
		},
		showProjectCloseMsg: function (cbId) {
			var title = this.getResourceBundle().getText("detailProjectClose");
			var text1 = this.getResourceBundle().getText("detailProjectCloseText1");
			var text2 = this.getResourceBundle().getText("detailProjectCloseText2");
			var oDialogMessage = new sap.m.Dialog({
				title: title,
				type: "Message",
				contentWidth: "600px",
				contentHeight: "200px",
				content: [
					new sap.m.Label({
						width: "100%",
						text: text1,
						textAlign: "Left"
					}),
					new sap.m.Label({
						width: "100%",
						text: text2,
						textAlign: "Left"
					})
				],
				buttons: [
					new sap.m.Button({
						text: "Cancel",
						type: "Reject",
						width: "150px",
						icon: "sap-icon://sys-cancel",
						press: function () {
							oDialogMessage.close();
							this.getView().byId("cbId").setSelected(false);
						}
					}),
					new sap.m.Button({
						text: "Close Project",
						type: "Accept",
						width: "150px",
						icon: "sap-icon://complete",
						press: function () {
							oDialogMessage.close();
							MessageToast.show("Project closed successfully");
						}
					})
				],
				afterClose: function () {
					oDialogMessage.destroy();
				}
			});
			oDialogMessage.open();
		}

	});
});